@@ -3,7 +3,6 @@ package ExtUtils::CBuilder::Base;
 use strict;
 use File::Spec;
 use File::Basename;
-use IO::File;
 use Config;
 use Text::ParseWords;
 
@@ -44,6 +43,28 @@ sub object_file {
   return "$file_base$self->{config}{obj_ext}";
 }
 
+sub arg_include_dirs {
+  my $self = shift;
+  return map {"-I$_"} @_;
+}
+
+sub arg_nolink { '-c' }
+
+sub arg_object_file {
+  my ($self, $file) = @_;
+  return ('-o', $file);
+}
+
+sub arg_share_object_file {
+  my ($self, $file) = @_;
+  return ($self->split_like_shell($self->{config}{lddlflags}), '-o', $file);
+}
+
+sub arg_exec_file {
+  my ($self, $file) = @_;
+  return ('-o', $file);
+}
+
 sub compile {
   my ($self, %args) = @_;
   die "Missing 'source' argument to compile()" unless defined $args{source};
@@ -52,18 +73,23 @@ sub compile {
 
   $args{object_file} ||= $self->object_file($args{source});
   
-  my @include_dirs = map {"-I$_"} (@{$args{include_dirs} || []},
-				   File::Spec->catdir($cf->{installarchlib}, 'CORE'));
+  my @include_dirs = $self->arg_include_dirs
+    (@{$args{include_dirs} || []},
+     File::Spec->catdir($cf->{installarchlib}, 'CORE'));
   
   my @extra_compiler_flags = $self->split_like_shell($args{extra_compiler_flags});
   my @cccdlflags = $self->split_like_shell($cf->{cccdlflags});
   my @ccflags = $self->split_like_shell($cf->{ccflags});
   my @optimize = $self->split_like_shell($cf->{optimize});
-  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags, '-c', @ccflags, @optimize);
+  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags,
+	       $self->arg_nolink,
+	       @ccflags, @optimize,
+	       $self->arg_object_file($args{object_file}),
+	      );
   
   my @cc = $self->split_like_shell($cf->{cc});
   
-  $self->do_system(@cc, @flags, '-o', $args{object_file}, $args{source})
+  $self->do_system(@cc, @flags, $args{source})
     or die "error building $args{object_file} from '$args{source}'";
 
   return $args{object_file};
@@ -75,8 +101,10 @@ sub have_compiler {
   
   my $tmpfile = File::Spec->catfile(File::Spec->tmpdir, 'compilet.c');
   {
-    my $fh = IO::File->new("> $tmpfile") or die "Can't create $tmpfile: $!";
-    print $fh "int boot_compilet() { return 1; }\n";
+    local *FH;
+    open FH, "> $tmpfile" or die "Can't create $tmpfile: $!";
+    print FH "int boot_compilet() { return 1; }\n";
+    close FH;
   }
 
   my ($obj_file, @lib_files);
@@ -100,6 +128,14 @@ sub lib_file {
   return "$dl_file.$self->{config}{dlext}";
 }
 
+
+sub exe_file {
+  my ($self, $dl_file) = @_;
+  $dl_file =~ s/\.[^.]+$//;
+  $dl_file =~ tr/"//d;
+  return "$dl_file$self->{config}{_exe}";
+}
+
 sub need_prelink { 0 }
 
 sub prelink {
@@ -124,11 +160,22 @@ sub prelink {
 
 sub link {
   my ($self, %args) = @_;
+  return $self->_do_link('lib_file', lddl => 1, %args);
+}
+
+sub link_executable {
+  my ($self, %args) = @_;
+  return $self->_do_link('exe_file', lddl => 0, %args);
+}
+				   
+sub _do_link {
+  my ($self, $type, %args) = @_;
+
   my $cf = $self->{config}; # For convenience
   
   my $objects = delete $args{objects};
   $objects = [$objects] unless ref $objects;
-  $args{lib_file} ||= $self->lib_file($objects->[0]);
+  my $out = $args{$type} || $self->$type($objects->[0]);
   
   my @temp_files;
   @temp_files =
@@ -136,15 +183,16 @@ sub link {
 		   dl_name => $args{module_name}) if $self->need_prelink;
   
   my @linker_flags = $self->split_like_shell($args{extra_linker_flags});
-  my @lddlflags = $self->split_like_shell($cf->{lddlflags});
+  my @output = $args{lddl} ? $self->arg_share_object_file($out) : $self->arg_exec_file($out);
   my @shrp = $self->split_like_shell($cf->{shrpenv});
   my @ld = $self->split_like_shell($cf->{ld});
-  $self->do_system(@shrp, @ld, @lddlflags, '-o', $args{lib_file}, @$objects, @linker_flags)
-    or die "error building $args{lib_file} from @$objects";
+  $self->do_system(@shrp, @ld, @output, @$objects, @linker_flags)
+    or die "error building $out from @$objects";
   
-  return wantarray ? ($args{lib_file}, @temp_files) : $args{lib_file};
+  return wantarray ? ($out, @temp_files) : $out;
 }
 
+
 sub do_system {
   my ($self, @cmd) = @_;
   print "@cmd\n";
